package com.ydd.gateway.filter;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.ydd.gateway.config.GatewayConfig;
import com.ydd.gateway.service.FilterService;
import com.ydd.gateway.service.GatewayContext;
import com.ydd.gateway.service.RecorderServerHttpRequestDecorator;
import com.ydd.gateway.util.StringUtils;
import io.netty.buffer.ByteBufAllocator;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilter;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.factory.AbstractGatewayFilterFactory;
import org.springframework.core.io.ByteArrayResource;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.core.io.buffer.DataBufferUtils;
import org.springframework.core.io.buffer.NettyDataBufferFactory;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpRequestDecorator;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.stereotype.Component;
import org.springframework.util.MultiValueMap;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.nio.CharBuffer;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.concurrent.atomic.AtomicReference;

/**
 @author Administrator
 *
 */
@Component
public class AuthorizeGatewayFilterFactory extends AbstractGatewayFilterFactory<AuthorizeGatewayFilterFactory.Config> {

    private static final Log logger = LogFactory.getLog(AuthorizeGatewayFilterFactory.class);

//	private static final String AUTHORIZE_TOKEN = "clientId";

    @Autowired
    FilterService filterService;

    public AuthorizeGatewayFilterFactory() {
        super(Config.class);
        logger.info("Loaded GatewayFilterFactory [Authorize]");
    }

    @Override
    public List<String> shortcutFieldOrder() {
        return Arrays.asList("enabled");
    }

    // url白名单
    public static final String URL = "/doc.html/**";
//    @Value("${waimai.meituan.developerId}")
//    public String aa ;
    @Override
    public GatewayFilter apply(AuthorizeGatewayFilterFactory.Config config) {
        return (exchange, chain) -> {
            if (!config.isEnabled()) {
                return chain.filter(exchange);
            }

            ServerHttpRequest request = exchange.getRequest();
            ServerHttpResponse response = exchange.getResponse();
            //	response.getHeaders().add("Content-Type", "text/plain;charset=UTF-8");

            // url白名单
            String url = request.getURI().getPath();
            logger.info(GatewayConfig.NACOS_SERVER_ADDR);
            String path = request.getPath().pathWithinApplication().value();
            System.out.println(path.substring(path.indexOf("/", 1), path.lastIndexOf("/")));
            System.out.println(URL.contains(path.substring(path.indexOf("/", 1), path.lastIndexOf("/"))));
//			if (URL.contains(path.substring(path.indexOf("/", 1), path.lastIndexOf("/")))) {
//				return chain.filter(exchange);
//			}
            HttpHeaders headers = request.getHeaders();
            String method = request.getMethodValue();
          //  String contentType = request.getHeaders().getFirst("Content-Type");
            MediaType contentType = headers.getContentType();
//            GatewayContext gatewayContext = exchange.getAttribute(GatewayContext.CACHE_GATEWAY_CONTEXT);
//            System.out.println(gatewayContext.getCacheBody());
//            System.out.println(gatewayContext.getFormData().toSingleValueMap());
//            System.out.println(gatewayContext.getAllRequestData().toSingleValueMap());
            if ("POST".equals(method)&&MediaType.APPLICATION_JSON.equals(contentType)){
                AtomicReference<String> requestBody = new AtomicReference<>("");
                RecorderServerHttpRequestDecorator requestDecorator = new RecorderServerHttpRequestDecorator(request);
                Flux<DataBuffer> body = requestDecorator.getBody();
                body.subscribe(buffer -> {
                    CharBuffer charBuffer = StandardCharsets.UTF_8.decode(buffer.asByteBuffer());
                    requestBody.set(charBuffer.toString());
                });
                //获取body参数
                JSONObject requestParams = JSONObject.parseObject(requestBody.get());
                System.out.println("**********************"+requestParams);
            }
            if(MediaType.APPLICATION_FORM_URLENCODED.equals(contentType)){
                GatewayContext gatewayContext = exchange.getAttribute(GatewayContext.CACHE_GATEWAY_CONTEXT);
                System.out.println(gatewayContext.getFormData());
            }
            if("GET".equals(method)){
                MultiValueMap<String, String> map =  request.getQueryParams();
                System.out.println("**********************"+JSONObject.toJSONString(map));
            }


//			String remoteIP = headers.getFirst("x-real-ip");
//			if (!StringUtils.hasText(remoteIP)) {
//				remoteIP = request.getRemoteAddress().getHostName();
//			}

            // 请求信息打印
           // logInfo(request, headers);
            TreeMap<String, String> params = new TreeMap<>();
            String signStr = headers.getFirst("sign");

            String sourceType = headers.getFirst("sourceType");
            String time = headers.getFirst("time");// 日期 yyyymmddHHmiss
            String nonce = headers.getFirst("nonce");// 随机数
            if (StringUtils.hasText(sourceType)) {
                params.put("sourceType", sourceType);
            }

            params.put("time", time);
            params.put("nonce", nonce);

            // 签名校验
            Mono<DataBuffer> mono = filterService.validateSign(params, signStr, response);
            if (mono != null) {
                //return response.writeWith(mono);
            }
            return chain.filter(exchange);
        };
    }

    public static class Config {

        // 控制是否开启认证
        private boolean enabled;

        public Config() {
        }

        public boolean isEnabled() {
            return enabled;
        }

        public void setEnabled(boolean enabled) {
            this.enabled = enabled;
        }

    }

    public void logInfo(ServerHttpRequest request, HttpHeaders headers) {

        String remoteIP = headers.getFirst("x-real-ip");
        if (!StringUtils.hasText(remoteIP)) {
            remoteIP = request.getRemoteAddress().getHostName();
        }
        String method = request.getMethodValue();
        String reqParams = JSON.toJSONString(request.getQueryParams(), SerializerFeature.DisableCircularReferenceDetect,
                SerializerFeature.WriteMapNullValue);
        String path = request.getPath().pathWithinApplication().value();
        logger.info("请求方的IP：" + remoteIP);
        logger.info("请求的接口：" + path);
        logger.info("请求的类型：" + method);
        logger.info("请求的参数：" + reqParams);

    }


}